import java.io.InputStream;
import java.net.URL;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Stock collection job for Google stock feeds, implementing an XML parsing routine.
 * 
 * @author fhogenboom
 */
public class StockJobGoogle extends StockJob
{

	private static Logger _log = LoggerFactory.getLogger(StockJobGoogle.class);

	/**
	 * Constructor.
	 */
	public StockJobGoogle()
	{
		super();
	}

	/**
	 * Retrieves the {@link org.slf4j.Logger} of the component.
	 * 
	 * @return {@link org.slf4j.Logger} used for logging system output.
	 */
	protected Logger getLogger()
	{
		return _log;
	}

	/**
	 * Parses feed containing stock data.
	 */
	protected void parseFeed()
	{
		// Initialize counter
		int count = 0;
		// Fetch data
		try
		{
			// Open feed
			InputStream file = new URL(stock_url).openStream();
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			// Parse XML document
			Document doc = db.parse(file);
			doc.getDocumentElement().normalize();
			// Get root elements
			NodeList nodeList1 = doc.getElementsByTagName(stock_xml_root);
			// For each element
			for (int i = 0; i < nodeList1.getLength(); i++)
			{
				Node node1 = nodeList1.item(i);
				// Add values to database
				dbs.addValue(stock_txt_date, date);
				dbs.addValue(stock_txt_time, time);
				// Update counter
				count += 2;
				// Get children
				NodeList nodeList2 = node1.getChildNodes();
				Iterator<Object> it = stock_xml_elms.iterator();
				// For each XML element
				while (it.hasNext())
				{
					boolean include = false;
					// Get field name
					String header = (String) it.next();
					// For each child
					for (int k = 0; k < nodeList2.getLength() & !include; k++)
					{
						Node node2 = nodeList2.item(k);
						if (node2.getNodeName().equals(header))
						{
							include = true;
						}
						// If field name is equal to XML element name
						if (include)
						{
							// Get value
							String value = node2.getAttributes().getNamedItem(stock_xml_attr).getNodeValue();
							// If empty value
							if (value.equals(""))
							{
								_log.debug("Empty XML element found for " + header);
								// Change to NaN
								value = "NaN";
							}
							// Add value (or NaN in case of empty value) to database
							dbs.addValue(header, value);
							// Update counter
							count++;
						}
					}
					// If field name not matched
					if (!include)
					{
						_log.debug("No XML element found for " + header);
						// Add NaN
						dbs.addValue(header, "NaN");
					}
				}
			}
		}
		catch (Exception e)
		{
			_log.error(e.toString());
		}
		_log.info("Collected " + count + " data points from " + stock_name);
	}

}